#include "init_periph.h"
#include "stm32f10x.h"


DMA_InitTypeDef           DMA_InitStructure;
extern tx_status_type tx_status;

uint8_t data_spi[17];


void init_all_periph(void)
{
 
  init_clk();
  init_gpio();
  init_uart1();
  timer1_init();
  timer3_init();
  timer16_init();
  init_adc();
  adc_dma_init();
  uart_dma_init();
}

void init_clk(void)
{
  //  
  //   HSI - 24MHZ
  ErrorStatus HSEStartUpStatus;
  
  RCC_DeInit(); /* RCC system reset(for debug purpose) */
  RCC_HSEConfig(RCC_HSE_ON);/* Enable HSE */
  HSEStartUpStatus = RCC_WaitForHSEStartUp();/* Wait till HSE is ready */

  if (HSEStartUpStatus == SUCCESS)
  {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);/* Enable Prefetch Buffer */
    FLASH_SetLatency(FLASH_Latency_2);/* Flash 2 wait state */
    
    RCC_HCLKConfig(RCC_SYSCLK_Div1);/* HCLK = SYSCLK */
    RCC_PCLK2Config(RCC_HCLK_Div1);/* PCLK2 = HCLK */
    RCC_PCLK1Config(RCC_HCLK_Div1);/* PCLK1 = HCLK */
    RCC_PREDIV1Config(RCC_PREDIV1_Source_HSE, RCC_PREDIV1_Div1);
    RCC_PLLConfig(RCC_PLLSource_PREDIV1, RCC_PLLMul_3);/* PLLCLK = 8MHz * 3 = 24 MHz <<<<<<<<<<<<<<<<<<<<<<<<<<*/
    RCC_PLLCmd(ENABLE);/* Enable PLL */
    while (RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET){};/* Wait till PLL is ready */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);/* Select PLL as system clock source */
    while (RCC_GetSYSCLKSource() != 0x08){}/* Wait till PLL is used as system clock source */
    
  }
  
  volatile uint32_t nCount;
  RCC_ClocksTypeDef RCC_Clocks;
  RCC_GetClocksFreq (&RCC_Clocks);
  nCount=RCC_Clocks.HCLK_Frequency;
    
  return;
  
}

void init_gpio(void)
{ 
  GPIO_InitTypeDef  GPIO_InitStructure;
  
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_GPIOB |RCC_APB2Periph_GPIOC| RCC_APB2Periph_AFIO, ENABLE);
  // -   
    
  GPIO_InitStructure.GPIO_Pin = SI_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;
  GPIO_Init(SI_PORT, &GPIO_InitStructure);
   
   //TIM1 channel1 PWM
  init_clk_pin(GPIO);

  GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable, ENABLE);
  
    //adc
  GPIO_InitStructure.GPIO_Pin   = ADC_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AIN;
  GPIO_Init(ADC_PORT, &GPIO_InitStructure);
  
  //HOLD
  GPIO_InitStructure.GPIO_Pin   = HOLD_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_OD;
  GPIO_Init(HOLD_PORT, &GPIO_InitStructure);
  disable_hold();
  
    // 
  GPIO_InitStructure.GPIO_Pin   = MOD_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
  GPIO_Init(MOD_PORT, &GPIO_InitStructure);
  
  // 
  GPIO_InitStructure.GPIO_Pin   = LED_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
  GPIO_Init(LED_PORT, &GPIO_InitStructure);
  
  //  
  GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_9 | GPIO_Pin_8;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  
  // 
  GPIO_InitStructure.GPIO_Pin   = SPEED_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN_FLOATING;
  GPIO_Init(SPEED_PORT, &GPIO_InitStructure);
  
  /*
  //TX pin - output - test
  GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  */
}





//  
void cap_start(uint32_t* pointer)
{
 
  tx_status = TRANSFER;
  init_clk_pin(GPIO);
  
  GPIO_SetBits(SI_PORT, SI_PIN);
  small_delay();
  GPIO_SetBits(CLK_PORT, CLK_PIN);
  small_delay();
  GPIO_ResetBits(SI_PORT, SI_PIN);
  small_delay();
  GPIO_ResetBits(CLK_PORT, CLK_PIN);
  
  init_clk_pin(PWM);
  
  dma_config(pointer);
  DMA_Cmd(DMA1_Channel1, ENABLE);
  TIM1->CR1 |= TIM_CR1_CEN;
}



//    ADC
// CLK 
void timer1_init(void)
{
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;
  
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);
  
  /* TIM1 configuration -------------------------------------------------------*/
  TIM_DeInit(TIM1);
  TIM_OCStructInit(&TIM_OCInitStructure);
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
  
  TIM_TimeBaseStructure.TIM_Prescaler = 0; 
  //TIM_TimeBaseStructure.TIM_Period = 80 - 1;
  TIM_TimeBaseStructure.TIM_Period = 70;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0x0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);
  
  // channel1 - PWM
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1; 
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;                
  //TIM_OCInitStructure.TIM_Pulse = 37; 
  TIM_OCInitStructure.TIM_Pulse = 35; 
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;         
  TIM_OC1Init(TIM1, &TIM_OCInitStructure);
  
  // channel2 - ADC trigger
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;// - 
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable; 
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputState_Disable;
  //TIM_OCInitStructure.TIM_Pulse = 55;
  TIM_OCInitStructure.TIM_Pulse = 55; 
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;         
  TIM_OC3Init(TIM1, &TIM_OCInitStructure);
  
  TIM_OC1PreloadConfig(TIM1, TIM_OCPreload_Enable);
  TIM_OC3PreloadConfig(TIM1, TIM_OCPreload_Enable);
  TIM_ARRPreloadConfig(TIM1, ENABLE); 
  //  

}

//   
//  input capture
//   
void timer3_init(void)
{
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  TIM_ICInitTypeDef  TIM_ICInitStructure;
  
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
  
  TIM_DeInit(TIM3);
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
  
  TIM_TimeBaseStructure.TIM_Prescaler = 299; 
  TIM_TimeBaseStructure.TIM_Period = 0xFFFF;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0x0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);
  
  // channel1 - input capture  
  TIM_ICInitStructure.TIM_Channel = TIM_Channel_1;
  TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Falling;
  TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
  TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
  TIM_ICInitStructure.TIM_ICFilter = 2;
  TIM_ICInit(TIM3, &TIM_ICInitStructure);
  
  
  TIM_ARRPreloadConfig(TIM3, ENABLE);
  TIM_ITConfig(TIM3, TIM_IT_CC1, ENABLE);
  NVIC_SetPriority(TIM3_IRQn, 1);
  NVIC_EnableIRQ(TIM3_IRQn);
  TIM_Cmd(TIM3, ENABLE);
}


//     (1  - 1 )
void timer16_init(void)
{
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;
  
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM16, ENABLE);
  
  TIM_DeInit(TIM16);
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
  
  TIM_TimeBaseStructure.TIM_Prescaler = 299; 
  TIM_TimeBaseStructure.TIM_Period = 0xFFFF;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0x0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM16, &TIM_TimeBaseStructure);
  
  
  // channel1 - PWM
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;                
  TIM_OCInitStructure.TIM_Pulse = 100; 
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;         
  TIM_OC1Init(TIM16, &TIM_OCInitStructure);
  
  TIM_ARRPreloadConfig(TIM16, ENABLE);
  TIM_OC1PreloadConfig(TIM16, TIM_OCPreload_Enable);
  
  TIM_ITConfig(TIM16, TIM_IT_CC1, ENABLE);
  NVIC_SetPriority(TIM1_UP_TIM16_IRQn, 2);
  NVIC_EnableIRQ(TIM1_UP_TIM16_IRQn);
  //     laser.c

}


void init_adc(void)
{
  ADC_InitTypeDef  ADC_InitStructure;
  
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1,ENABLE);
  RCC_ADCCLKConfig(RCC_PCLK2_Div2);
  
  /* ADC1 configuration ------------------------------------------------------*/
  ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_InitStructure.ADC_ScanConvMode = DISABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
  ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T1_CC3;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfChannel = 1;
  ADC_Init(ADC1, &ADC_InitStructure);
  
  ADC_ExternalTrigConvCmd(ADC1, ENABLE);
  ADC_DMACmd(ADC1, ENABLE);//adc    DMA  
  
  //ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_1Cycles5);
  ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_7Cycles5);
  
  /* Enable ADC1 */
  ADC_Cmd(ADC1, ENABLE);
  
  ADC_ResetCalibration(ADC1);
  while(ADC_GetResetCalibrationStatus(ADC1)){};
  ADC_StartCalibration(ADC1);   
  while(ADC_GetCalibrationStatus(ADC1)){};

}

//     
void adc_dma_init(void)
{
  DMA_InitTypeDef           DMA_InitStructure;
  
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
  
  DMA_DeInit(DMA1_Channel1);
  DMA_InitStructure.DMA_PeripheralBaseAddr = ADC1_DR_Address;
  DMA_InitStructure.DMA_MemoryBaseAddr = 0;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
  DMA_InitStructure.DMA_BufferSize = 128;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  
  DMA_Init(DMA1_Channel1, &DMA_InitStructure); 
  DMA_Cmd(DMA1_Channel1, ENABLE);
  
  DMA_ITConfig(DMA1_Channel1, DMA_IT_TC, ENABLE);//transfer complete
  
  NVIC_SetPriority(DMA1_Channel1_IRQn, 4);
  NVIC_EnableIRQ(DMA1_Channel1_IRQn);
}




void dma_config(uint32_t* pointer )
{ 
   DMA_Cmd(DMA1_Channel1, DISABLE);
   DMA1_Channel1->CMAR = (uint32_t)pointer;
   DMA1_Channel1->CNDTR = 128;
}


// DMA  UART
void uart_dma_init(void)
{
  DMA_InitTypeDef           DMA_InitStructure;
  
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
  
  DMA_DeInit(DMA1_Channel4);
  DMA_InitStructure.DMA_PeripheralBaseAddr =  (uint32_t)&USART1->DR;
  DMA_InitStructure.DMA_MemoryBaseAddr = 0;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStructure.DMA_BufferSize = 128;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_PeripheralDataSize_Byte;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStructure.DMA_Priority = DMA_Priority_Medium;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  
  DMA_Init(DMA1_Channel4, &DMA_InitStructure); 
  DMA_Cmd(DMA1_Channel4, ENABLE);
  
  //  
}


void small_delay(void)
{
asm("nop");
asm("nop");asm("nop");
asm("nop");
  asm("nop");
asm("nop");asm("nop");
asm("nop");
}


void enable_laser(void)
{
  MOD_PORT->ODR |= MOD_PIN;
}

void disable_laser(void)
{
  MOD_PORT->ODR&= ~MOD_PIN;
}

void enable_hold(void)
{
   HOLD_PORT->ODR |= HOLD_PIN;
}

void disable_hold(void)
{
  HOLD_PORT->ODR&= ~HOLD_PIN;
}


void init_clk_pin(clk_pin_type pin_type)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  
  GPIO_InitStructure.GPIO_Pin = CLK_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  
  if (pin_type == PWM)
  {    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;}
  else
  {    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;}
  
  GPIO_Init(CLK_PORT, &GPIO_InitStructure);

}

void switch_led(uint8_t val)
{
  if (val != 0) {LED_PORT->ODR|= LED_PIN;} else {LED_PORT->ODR&= ~LED_PIN;}
}





void init_uart1(void)
{
      GPIO_InitTypeDef  GPIO_InitStructure;
      USART_InitTypeDef USART_InitStructure;

      /* Enable GPIOA clock                                                   */
      RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_USART1, ENABLE);

      /* Configure USART1 Tx (PA9) as alternate function push-pull            */
      GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_9;
      GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
      GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP;
      GPIO_Init(GPIOA, &GPIO_InitStructure);
      
      
      USART_InitStructure.USART_BaudRate = 38400;   
      USART_InitStructure.USART_WordLength = USART_WordLength_8b;   
      USART_InitStructure.USART_StopBits = USART_StopBits_1;   
      USART_InitStructure.USART_Parity = USART_Parity_No ;   
      USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;   
      USART_InitStructure.USART_Mode = USART_Mode_Tx; 

      USART_Init(USART1, &USART_InitStructure);
      
      USART_DMACmd(USART1, USART_DMAReq_Tx, ENABLE);
      
      USART_Cmd(USART1, ENABLE);
}



